<?php

namespace Oro\Bundle\CustomerBundle\Tests\Functional\Api;

use Oro\Bundle\AddressBundle\Tests\Functional\Api\RestJsonApi\AddressCountryAndRegionTestTrait;
use Oro\Bundle\AddressBundle\Tests\Functional\Api\RestJsonApi\PrimaryAddressTestTrait;
use Oro\Bundle\AddressBundle\Tests\Functional\Api\RestJsonApi\UnchangeableAddressOwnerTestTrait;
use Oro\Bundle\ApiBundle\Tests\Functional\RestJsonApiTestCase;
use Oro\Bundle\CustomerBundle\Entity\CustomerUser;
use Oro\Bundle\CustomerBundle\Entity\CustomerUserAddress;
use Oro\Bundle\CustomerBundle\Entity\CustomerUserRole;

/**
 * @dbIsolationPerTest
 * @SuppressWarnings(PHPMD.TooManyMethods)
 * @SuppressWarnings(PHPMD.TooManyPublicMethods)
 * @SuppressWarnings(PHPMD.ExcessivePublicCount)
 * @SuppressWarnings(PHPMD.ExcessiveClassLength)
 * @SuppressWarnings(PHPMD.ExcessiveClassComplexity)
 */
class CustomerUserAddressTest extends RestJsonApiTestCase
{
    use AddressCountryAndRegionTestTrait;
    use PrimaryAddressTestTrait;
    use UnchangeableAddressOwnerTestTrait;
    use AddressTypeTestTrait;

    private const ENTITY_CLASS                     = CustomerUserAddress::class;
    private const ENTITY_TYPE                      = 'customer_user_addresses';
    private const OWNER_ENTITY_TYPE                = 'customer_users';
    private const OWNER_RELATIONSHIP               = 'frontendOwner';
    private const CREATE_MIN_REQUEST_DATA          = 'create_customer_user_address_min.yml';
    private const OWNER_CREATE_MIN_REQUEST_DATA    = 'create_customer_user_min.yml';
    private const IS_REGION_REQUIRED               = true;
    private const COUNTRY_REGION_ADDRESS_REF       = 'other.user@test.com.address_1';
    private const PRIMARY_ADDRESS_REF              = 'other.user@test.com.address_1';
    private const DEFAULT_ADDRESS_REF              = 'other.user@test.com.address_1';
    private const BILLING_ADDRESS_REF              = 'grzegorz.brzeczyszczykiewicz@example.com.address_2';
    private const BILLING_AND_SHIPPING_ADDRESS_REF = 'grzegorz.brzeczyszczykiewicz@example.com.address_1';
    private const UNCHANGEABLE_ADDRESS_REF         = 'grzegorz.brzeczyszczykiewicz@example.com.address_1';
    private const ANOTHER_OWNER_REF                = 'other.user@test.com';
    private const ANOTHER_OWNER_ADDRESS_2_REF      = 'other.user@test.com.address_2';

    protected function setUp()
    {
        parent::setUp();

        $this->loadFixtures(['@OroCustomerBundle/Tests/Functional/Api/DataFixtures/customer_user_addresses.yml']);
        $role = $this->getEntityManager()
            ->getRepository(CustomerUserRole::class)
            ->findOneBy(['role' => 'ROLE_FRONTEND_ADMINISTRATOR']);
        $this->getReferenceRepository()->addReference('ROLE_FRONTEND_ADMINISTRATOR', $role);
    }

    /**
     * @param CustomerUserAddress $address
     *
     * @return CustomerUser
     */
    private function getOwner(CustomerUserAddress $address)
    {
        return $address->getFrontendOwner();
    }

    public function testGetList()
    {
        $response = $this->cget(
            ['entity' => self::ENTITY_TYPE]
        );

        $this->assertResponseContains('cget_customer_user_address.yml', $response);
    }

    public function testGet()
    {
        $response = $this->get(
            ['entity' => self::ENTITY_TYPE, 'id' => '<toString(@other.user@test.com.address_1->id)>']
        );

        $this->assertResponseContains('get_customer_user_address.yml', $response);
    }

    public function testCreate()
    {
        $customer = $this->getReference('customer.1');
        $ownerId = $customer->getOwner()->getId();
        $organizationId = $customer->getOrganization()->getId();
        $customerUserId = $this->getReference('other.user@test.com')->getId();
        $countryId = $this->getReference('country.US')->getIso2Code();
        $regionId = $this->getReference('region.US-NY')->getCombinedCode();

        $response = $this->post(
            ['entity' => self::ENTITY_TYPE],
            'create_customer_user_address.yml'
        );

        $addressId = (int)$this->getResourceId($response);
        $responseContent = $this->updateResponseContent('create_customer_user_address.yml', $response);
        $this->assertResponseContains($responseContent, $response);

        /** @var CustomerUserAddress $address */
        $address = $this->getEntityManager()
            ->find(self::ENTITY_CLASS, $addressId);
        self::assertNotNull($address);
        self::assertEquals('New Address', $address->getLabel());
        self::assertTrue($address->isPrimary());
        self::assertEquals('123-456', $address->getPhone());
        self::assertEquals('Acme', $address->getOrganization());
        self::assertEquals('Street 1', $address->getStreet());
        self::assertEquals('Street 2', $address->getStreet2());
        self::assertEquals('Los Angeles', $address->getCity());
        self::assertEquals('90001', $address->getPostalCode());
        self::assertEquals('Mr.', $address->getNamePrefix());
        self::assertEquals('M.D.', $address->getNameSuffix());
        self::assertEquals('John', $address->getFirstName());
        self::assertEquals('Edgar', $address->getMiddleName());
        self::assertEquals('Doo', $address->getLastName());
        self::assertCount(2, $address->getAddressTypes());
        foreach ($address->getAddressTypes() as $type) {
            if ('billing' === $type->getType()->getName()) {
                self::assertTrue($type->isDefault(), 'billing default');
            } else {
                self::assertEquals('shipping', $type->getType()->getName(), 'shipping type');
                self::assertFalse($type->isDefault(), 'shipping default');
            }
        }
        self::assertEquals($organizationId, $address->getSystemOrganization()->getId());
        self::assertEquals($ownerId, $address->getOwner()->getId());
        self::assertEquals($customerUserId, $address->getFrontendOwner()->getId());
        self::assertEquals($countryId, $address->getCountry()->getIso2Code());
        self::assertEquals($regionId, $address->getRegion()->getCombinedCode());
    }

    public function testTryToCreateWithRequiredDataOnlyAndWithoutOrganizationAndFirstNameAndLastName()
    {
        $data = $this->getRequestData(self::CREATE_MIN_REQUEST_DATA);
        unset($data['data']['attributes']['organization']);
        $response = $this->post(
            ['entity' => self::ENTITY_TYPE],
            $data,
            [],
            false
        );

        $this->assertResponseValidationErrors(
            [
                [
                    'title'  => 'name or organization constraint',
                    'detail' => 'Organization or First Name and Last Name should not be blank.',
                    'source' => ['pointer' => '/data/attributes/organization']
                ],
                [
                    'title'  => 'name or organization constraint',
                    'detail' => 'First Name and Last Name or Organization should not be blank.',
                    'source' => ['pointer' => '/data/attributes/firstName']
                ],
                [
                    'title'  => 'name or organization constraint',
                    'detail' => 'Last Name and First Name or Organization should not be blank.',
                    'source' => ['pointer' => '/data/attributes/lastName']
                ]
            ],
            $response
        );
    }

    public function testCreateWithRequiredDataOnlyAndOrganization()
    {
        $customer = $this->getReference('customer.1');
        $ownerId = $customer->getOwner()->getId();
        $organizationId = $customer->getOrganization()->getId();
        $customerUserId = $this->getReference('other.user@test.com')->getId();
        $countryId = $this->getReference('country.US')->getIso2Code();
        $regionId = $this->getReference('region.US-NY')->getCombinedCode();

        $data = $this->getRequestData(self::CREATE_MIN_REQUEST_DATA);
        $response = $this->post(
            ['entity' => self::ENTITY_TYPE],
            $data
        );

        $addressId = (int)$this->getResourceId($response);
        $responseContent = $data;
        $responseContent['data']['attributes']['label'] = null;
        $responseContent['data']['attributes']['primary'] = false;
        $responseContent['data']['attributes']['phone'] = null;
        $responseContent['data']['attributes']['street2'] = null;
        $responseContent['data']['attributes']['namePrefix'] = null;
        $responseContent['data']['attributes']['nameSuffix'] = null;
        $responseContent['data']['attributes']['firstName'] = null;
        $responseContent['data']['attributes']['middleName'] = null;
        $responseContent['data']['attributes']['lastName'] = null;
        $responseContent['data']['attributes']['types'] = [];
        $responseContent['data']['relationships']['owner']['data'] = [
            'type' => 'users',
            'id'   => (string)$ownerId
        ];
        $responseContent['data']['relationships']['systemOrganization']['data'] = [
            'type' => 'organizations',
            'id'   => (string)$organizationId
        ];
        $this->assertResponseContains($responseContent, $response);

        /** @var CustomerUserAddress $address */
        $address = $this->getEntityManager()
            ->find(self::ENTITY_CLASS, $addressId);
        self::assertNotNull($address);
        self::assertNull($address->getLabel());
        self::assertFalse($address->isPrimary());
        self::assertNull($address->getPhone());
        self::assertEquals('Acme', $address->getOrganization());
        self::assertEquals('Street 1', $address->getStreet());
        self::assertNull($address->getStreet2());
        self::assertEquals('Los Angeles', $address->getCity());
        self::assertEquals('90001', $address->getPostalCode());
        self::assertNull($address->getNamePrefix());
        self::assertNull($address->getNameSuffix());
        self::assertNull($address->getFirstName());
        self::assertNull($address->getMiddleName());
        self::assertNull($address->getLastName());
        self::assertCount(0, $address->getAddressTypes());
        self::assertEquals($organizationId, $address->getSystemOrganization()->getId());
        self::assertEquals($ownerId, $address->getOwner()->getId());
        self::assertEquals($customerUserId, $address->getFrontendOwner()->getId());
        self::assertEquals($countryId, $address->getCountry()->getIso2Code());
        self::assertEquals($regionId, $address->getRegion()->getCombinedCode());
    }

    public function testCreateWithRequiredDataOnlyAndFirstNameAndLastName()
    {
        $customer = $this->getReference('customer.1');
        $ownerId = $customer->getOwner()->getId();
        $organizationId = $customer->getOrganization()->getId();
        $customerUserId = $this->getReference('other.user@test.com')->getId();
        $countryId = $this->getReference('country.US')->getIso2Code();
        $regionId = $this->getReference('region.US-NY')->getCombinedCode();

        $data = $this->getRequestData(self::CREATE_MIN_REQUEST_DATA);
        unset($data['data']['attributes']['organization']);
        $data['data']['attributes']['firstName'] = 'John';
        $data['data']['attributes']['lastName'] = 'Doo';
        $response = $this->post(
            ['entity' => self::ENTITY_TYPE],
            $data
        );

        $addressId = (int)$this->getResourceId($response);
        $responseContent = $data;
        $responseContent['data']['attributes']['label'] = null;
        $responseContent['data']['attributes']['primary'] = false;
        $responseContent['data']['attributes']['phone'] = null;
        $responseContent['data']['attributes']['organization'] = null;
        $responseContent['data']['attributes']['street2'] = null;
        $responseContent['data']['attributes']['namePrefix'] = null;
        $responseContent['data']['attributes']['nameSuffix'] = null;
        $responseContent['data']['attributes']['middleName'] = null;
        $responseContent['data']['attributes']['types'] = [];
        $responseContent['data']['relationships']['owner']['data'] = [
            'type' => 'users',
            'id'   => (string)$ownerId
        ];
        $responseContent['data']['relationships']['systemOrganization']['data'] = [
            'type' => 'organizations',
            'id'   => (string)$organizationId
        ];
        $this->assertResponseContains($responseContent, $response);

        /** @var CustomerUserAddress $address */
        $address = $this->getEntityManager()
            ->find(self::ENTITY_CLASS, $addressId);
        self::assertNotNull($address);
        self::assertNull($address->getLabel());
        self::assertFalse($address->isPrimary());
        self::assertNull($address->getPhone());
        self::assertNull($address->getOrganization());
        self::assertEquals('Street 1', $address->getStreet());
        self::assertNull($address->getStreet2());
        self::assertEquals('Los Angeles', $address->getCity());
        self::assertEquals('90001', $address->getPostalCode());
        self::assertNull($address->getNamePrefix());
        self::assertNull($address->getNameSuffix());
        self::assertEquals('John', $address->getFirstName());
        self::assertNull($address->getMiddleName());
        self::assertEquals('Doo', $address->getLastName());
        self::assertCount(0, $address->getAddressTypes());
        self::assertEquals($organizationId, $address->getSystemOrganization()->getId());
        self::assertEquals($ownerId, $address->getOwner()->getId());
        self::assertEquals($customerUserId, $address->getFrontendOwner()->getId());
        self::assertEquals($countryId, $address->getCountry()->getIso2Code());
        self::assertEquals($regionId, $address->getRegion()->getCombinedCode());
    }

    public function testTryToCreateWithoutCustomerUser()
    {
        $data = $this->getRequestData(self::CREATE_MIN_REQUEST_DATA);
        unset($data['data']['relationships']['frontendOwner']);
        $response = $this->post(
            ['entity' => self::ENTITY_TYPE],
            $data,
            [],
            false
        );

        $this->assertResponseValidationError(
            [
                'title'  => 'not blank constraint',
                'detail' => 'This value should not be blank.',
                'source' => ['pointer' => '/data/relationships/frontendOwner/data']
            ],
            $response
        );
    }

    public function testTryToCreateWithNullCustomerUser()
    {
        $data = $this->getRequestData(self::CREATE_MIN_REQUEST_DATA);
        $data['data']['relationships']['frontendOwner']['data'] = null;
        $response = $this->post(
            ['entity' => self::ENTITY_TYPE],
            $data,
            [],
            false
        );

        $this->assertResponseValidationError(
            [
                'title'  => 'not blank constraint',
                'detail' => 'This value should not be blank.',
                'source' => ['pointer' => '/data/relationships/frontendOwner/data']
            ],
            $response
        );
    }

    public function testUpdate()
    {
        $response = $this->patch(
            ['entity' => self::ENTITY_TYPE, 'id' => '<toString(@other.user@test.com.address_1->id)>'],
            'update_customer_user_address.yml'
        );

        $this->assertResponseContains('update_customer_user_address.yml', $response);

        /** @var CustomerUserAddress $address */
        $address = $this->getEntityManager()
            ->find(self::ENTITY_CLASS, $this->getResourceId($response));

        self::assertEquals('City updated', $address->getCity());
        self::assertEquals('First name updated', $address->getFirstName());
        self::assertEquals('Address label updated', $address->getLabel());
        self::assertEquals(
            $this->getReference('other.user@test.com')->getId(),
            $address->getFrontendOwner()->getId()
        );
        self::assertEquals(
            $this->getReference('country.US')->getIso2Code(),
            $address->getCountry()->getIso2Code()
        );
    }

    public function testDelete()
    {
        $addressId = $this->getReference('other.user@test.com.address_1')->getId();

        $this->delete(
            ['entity' => self::ENTITY_TYPE, 'id' => $addressId]
        );

        $address = $this->getEntityManager()
            ->find(self::ENTITY_CLASS, $addressId);
        self::assertTrue(null === $address);
    }

    public function testDeleteList()
    {
        $addressId = $this->getReference('other.user@test.com.address_1')->getId();

        $this->cdelete(
            ['entity' => self::ENTITY_TYPE],
            ['filter' => ['id' => (string)$addressId]]
        );

        $address = $this->getEntityManager()
            ->find(self::ENTITY_CLASS, $addressId);
        self::assertTrue(null === $address);
    }

    public function testTryToSetNullCountry()
    {
        $addressId = $this->getReference('other.user@test.com.address_1')->getId();
        $data = [
            'data' => [
                'type'          => self::ENTITY_TYPE,
                'id'            => (string)$addressId,
                'relationships' => [
                    'country' => [
                        'data' => null
                    ]
                ]
            ]
        ];
        $response = $this->patch(
            ['entity' => self::ENTITY_TYPE, 'id' => $addressId],
            $data,
            [],
            false
        );

        $this->assertResponseValidationError(
            [
                'title'  => 'not blank constraint',
                'detail' => 'This value should not be blank.',
                'source' => ['pointer' => '/data/relationships/country/data']
            ],
            $response
        );
    }

    public function testTryToUpdateCustomerUser()
    {
        $addressId = $this->getReference('other.user@test.com.address_1')->getId();

        $response = $this->patch(
            ['entity' => self::ENTITY_TYPE, 'id' => $addressId],
            [
                'data' => [
                    'type'          => self::ENTITY_TYPE,
                    'id'            => (string)$addressId,
                    'relationships' => [
                        'frontendOwner' => [
                            'data' => [
                                'type' => 'customer_users',
                                'id'   => '<toString(@other.user@test.com->id)>'
                            ]
                        ]
                    ]
                ]
            ],
            [],
            false
        );

        $this->assertResponseValidationError(
            [
                'title'  => 'extra fields constraint',
                'detail' => 'This form should not contain extra fields: "frontendOwner"'
            ],
            $response
        );
    }

    public function testTryToUpdateCustomerUserViaRelationship()
    {
        $addressId = $this->getReference('other.user@test.com.address_1')->getId();
        $data = [
            'data' => [
                'type' => 'customer_users',
                'id'   => '<toString(@other.user@test.com->id)>'
            ]
        ];

        $response = $this->patchRelationship(
            ['entity' => self::ENTITY_TYPE, 'id' => (string)$addressId, 'association' => 'frontendOwner'],
            $data,
            [],
            false
        );

        self::assertMethodNotAllowedResponse($response, 'GET');
    }

    public function testTryToUpdateSystemOrganization()
    {
        $addressId = $this->getReference('other.user@test.com.address_1')->getId();

        $response = $this->patch(
            ['entity' => self::ENTITY_TYPE, 'id' => $addressId],
            [
                'data' => [
                    'type'          => self::ENTITY_TYPE,
                    'id'            => (string)$addressId,
                    'relationships' => [
                        'systemOrganization' => [
                            'data' => [
                                'type' => 'organizations',
                                'id'   => '<toString(@organization->id)>'
                            ]
                        ]
                    ]
                ]
            ],
            [],
            false
        );

        $this->assertResponseValidationError(
            [
                'title'  => 'extra fields constraint',
                'detail' => 'This form should not contain extra fields: "systemOrganization"'
            ],
            $response
        );
    }

    public function testTryToUpdateSystemOrganizationViaRelationship()
    {
        $addressId = $this->getReference('other.user@test.com.address_1')->getId();
        $data = [
            'data' => [
                'type' => 'organizations',
                'id'   => '<toString(@organization->id)>'
            ]
        ];

        $response = $this->patchRelationship(
            ['entity' => self::ENTITY_TYPE, 'id' => (string)$addressId, 'association' => 'systemOrganization'],
            $data,
            [],
            false
        );

        self::assertMethodNotAllowedResponse($response, 'GET');
    }
}
